# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os

from itertools import product
from subprocess import PIPE

from law.util import colored

from analysis.tasks.FnloCppreadFinal import FnloCppreadFinal
from analysis.tasks.Recombine import Recombine

from analysis.framework import Task
from analysis.util import interruptable_popen

logger = logging.getLogger(__name__)


class KfacComparison(Task, law.LocalWorkflow):
    command = 'fastnnlo_kfaccomp.py'

    merge_dir = luigi.Parameter(default='grids')
    plots_dir = luigi.Parameter(default='plots')

    orders = luigi.ListParameter()
    observables = luigi.ListParameter()

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output directory."""
        return {
            i : {'observable': obs}
            for i, obs in enumerate(self.observables)
        }

    def run_configs(self):
        """Run configuration dicts. `run` method loops through these to populate the output directory."""
        return [
            {'order': order}
            for order in self.orders
        ]

    def workflow_requires(self):
        return {'fnlocppread': FnloCppreadFinal(), 'recombine': Recombine()}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        return law.LocalDirectoryTarget(
            '{}/{}/{}/{}/{}'.format(
                out_dir,
                self.plots_dir, self.name, self.__class__.__name__,
                self.branch_data['observable']
            )
        )

    def run(self):
        try:
            with self.output().temporary_path() as self.temp_output_path:
                os.mkdir(self.temp_output_path)

                for i, run_config in enumerate(self.run_configs()):
                    datfile = '{}/{}/Combined/Final/{}.{}.{}.dat'.format(
                        self.merge_dir, self.name, self.process,
                        run_config['order'],
                        self.branch_data['observable']
                    )
                    outfile = '{}/{}.{}.{}'.format(
                        self.temp_output_path, self.process,
                        self.branch_data['observable'],
                        run_config['order'],
                        self.branch_data['observable'],
                    )

                    command_line_args = [
                        self.command, datfile, '-f', outfile
                    ]

                    logger.info("Running command [%s/%s]: %s", i+1,
                        len(self.run_configs()),
                        colored(' '.join(command_line_args), 'yellow'))
                    code, out, error = interruptable_popen(command_line_args,
                                                           stdout=PIPE,
                                                           stderr=PIPE)

                    logger.debug("Return code: %d", code)
                    logger.debug("Standard output:\n%s\n", out)
                    logger.debug("Standard error:\n%s\n", error)

                    if (code == 0):
                        logger.info("%s status: SUCCESS", self.command)
                    else:
                        raise RuntimeError("{} status: FAIL (exit code {})".format(self.command, code))

        # mark job as failed on error, make log entry and reraise
        except Exception as e:
            os.rmdir(self.temp_output_path)
            logger.error('%s raised during job execution: %s', e.__class__.__name__, e)
            raise
